@@ -23,12 +23,11 @@
 #include "Sprites/skinSpritesData.h"
 #include "goal.h"
 
-#define KDE_NO_COMPAT
+#include <QApplication>
 #include <QFile>
 #include <qdom.h>
 #include <QPainter>
 #include <QPixmapCache>
-#include <QFileInfo>
 #include <QMenuBar>
 #include <KLocalizedString>
 #include "ksirk_debug.h"
@@ -57,16 +56,6 @@ ONU::ONU(GameAutomaton* automaton,
   m_zoomFactorFinal(1)
 {
   qCDebug(KSIRK_LOG) << "ONU constructor: " << m_configFileName;
-  QFileInfo qfi(m_configFileName);
-  qCDebug(KSIRK_LOG) << "skin written at :" << qfi.lastModified().toSecsSinceEpoch();
-  //qCDebug(KSIRK_LOG) << "cache created at:" << m_automaton->pixmapCache().timestamp();
-#pragma message("port to qt5")
-#if 0
-  if (m_automaton->pixmapCache().timestamp() < qfi.lastModified().toSecsSinceEpoch())
-  {
-    m_automaton->pixmapCache().discard();
-  }
-#endif
   m_font.family = "URW Chancery L";
   m_font.size = (int)(13*m_zoom);
   m_font.weight = QFont::Bold;
@@ -82,7 +71,7 @@ ONU::ONU(GameAutomaton* automaton,
 //   unsigned int continentId = 0;
   KConfig config(configFileName);
 
-  KConfigGroup onugroup = config.group("onu");
+  KConfigGroup onugroup = config.group(QStringLiteral("onu"));
 
   qCDebug(KSIRK_LOG) << "ONU XML format version: " << onugroup.readEntry("format-version");
   QString formatVersion = onugroup.readEntry("format-version");
@@ -100,7 +89,8 @@ ONU::ONU(GameAutomaton* automaton,
   m_skin = onugroup.readEntry("skinpath");
 
   qCDebug(KSIRK_LOG) << "skin snapshot file: " << QStandardPaths::locate(QStandardPaths::AppDataLocation, m_skin + "/Images/snapshot.jpg");
-  if (!QPixmapCache::find(m_skin+"snapshot", &m_snapshot))
+  const QString snapshotCacheId = m_skin + QLatin1String("snapshot");
+  if (!QPixmapCache::find(snapshotCacheId, &m_snapshot))
   {
     // Pixmap isn't in the cache, create it and insert to cache
     m_snapshot = QPixmap(QStandardPaths::locate(QStandardPaths::AppDataLocation, m_skin + "/Images/snapshot.jpg"));
@@ -108,7 +98,7 @@ ONU::ONU(GameAutomaton* automaton,
     {
       qCCritical(KSIRK_LOG) << "Was not able to load the snapshot image: " << QStandardPaths::locate(QStandardPaths::AppDataLocation, m_skin + "/Images/snapshot.jpg");
     }
-    QPixmapCache::insert(m_skin+"snapshot", m_snapshot);
+    QPixmapCache::insert(snapshotCacheId, m_snapshot);
   }
   m_width  = onugroup.readEntry("width",0);
   m_height  = onugroup.readEntry("height",0);
@@ -149,7 +139,8 @@ ONU::ONU(GameAutomaton* automaton,
   }
   qCDebug(KSIRK_LOG) << "Loading map mask file: " << mapMaskFileName;
   QPixmap countriesMaskPix;
-  if (!QPixmapCache::find(m_skin+"mapmask", &countriesMaskPix))
+  const QString mapMaskCacheId = m_skin + QLatin1String("mapmask");
+  if (!QPixmapCache::find(mapMaskCacheId, &countriesMaskPix))
   {
     // Pixmap isn't in the cache, create it and insert to cache
     countriesMaskPix = QPixmap(mapMaskFileName);
@@ -157,67 +148,67 @@ ONU::ONU(GameAutomaton* automaton,
     {
       qCCritical(KSIRK_LOG) << "Was not able to load the map mask image: " << mapMaskFileName ;
     }
-    QPixmapCache::insert(m_skin+"mapmask", countriesMaskPix);
+    QPixmapCache::insert(mapMaskCacheId, countriesMaskPix);
   }
 //   countriesMask = QImage(mapMaskFileName);
   countriesMask = countriesMaskPix.toImage();
   
   Sprites::SkinSpritesData::changeable().intData("width-between-flag-and-fighter", onugroup.readEntry("width-between-flag-and-fighter",0));
 
-  Sprites::SkinSpritesData::changeable().intData("flag-width", config.group("flag").readEntry("width",0));
-  Sprites::SkinSpritesData::changeable().intData("flag-height", config.group("flag").readEntry("height",0));
-  Sprites::SkinSpritesData::changeable().intData("flag-frames", config.group("flag").readEntry("frames",0));
-  Sprites::SkinSpritesData::changeable().intData("flag-versions", config.group("flag").readEntry("versions",0));
+  Sprites::SkinSpritesData::changeable().intData("flag-width", config.group(QStringLiteral("flag")).readEntry("width",0));
+  Sprites::SkinSpritesData::changeable().intData("flag-height", config.group(QStringLiteral("flag")).readEntry("height",0));
+  Sprites::SkinSpritesData::changeable().intData("flag-frames", config.group(QStringLiteral("flag")).readEntry("frames",0));
+  Sprites::SkinSpritesData::changeable().intData("flag-versions", config.group(QStringLiteral("flag")).readEntry("versions",0));
 
-//   Sprites::SkinSpritesData::changeable().strData("infantry-id", config.group("infantry").readEntry("id"));
-  Sprites::SkinSpritesData::changeable().intData("infantry-width", config.group("infantry").readEntry("width",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry-height", config.group("infantry").readEntry("height",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry-frames", config.group("infantry").readEntry("frames",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry-versions", config.group("infantry").readEntry("versions",0));
+//   Sprites::SkinSpritesData::changeable().strData("infantry-id", config.group(QStringLiteral("infantry")).readEntry("id"));
+  Sprites::SkinSpritesData::changeable().intData("infantry-width", config.group(QStringLiteral("infantry")).readEntry("width",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry-height", config.group(QStringLiteral("infantry")).readEntry("height",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry-frames", config.group(QStringLiteral("infantry")).readEntry("frames",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry-versions", config.group(QStringLiteral("infantry")).readEntry("versions",0));
 
-//   Sprites::SkinSpritesData::changeable().strData("infantry1-id", config.group("infantry1").readEntry("id"));
-  Sprites::SkinSpritesData::changeable().intData("infantry1-width", config.group("infantry1").readEntry("width",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry1-height", config.group("infantry1").readEntry("height",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry1-frames", config.group("infantry1").readEntry("frames",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry1-versions", config.group("infantry1").readEntry("versions",0));
+//   Sprites::SkinSpritesData::changeable().strData("infantry1-id", config.group(QStringLiteral("infantry1")).readEntry("id"));
+  Sprites::SkinSpritesData::changeable().intData("infantry1-width", config.group(QStringLiteral("infantry1")).readEntry("width",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry1-height", config.group(QStringLiteral("infantry1")).readEntry("height",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry1-frames", config.group(QStringLiteral("infantry1")).readEntry("frames",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry1-versions", config.group(QStringLiteral("infantry1")).readEntry("versions",0));
 
-//   Sprites::SkinSpritesData::changeable().strData("infantry2-id", config.group("infantry2").readEntry("id"));
-  Sprites::SkinSpritesData::changeable().intData("infantry2-width", config.group("infantry2").readEntry("width",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry2-height", config.group("infantry2").readEntry("height",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry2-frames", config.group("infantry2").readEntry("frames",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry2-versions", config.group("infantry2").readEntry("versions",0));
+//   Sprites::SkinSpritesData::changeable().strData("infantry2-id", config.group(QStringLiteral("infantry2")).readEntry("id"));
+  Sprites::SkinSpritesData::changeable().intData("infantry2-width", config.group(QStringLiteral("infantry2")).readEntry("width",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry2-height", config.group(QStringLiteral("infantry2")).readEntry("height",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry2-frames", config.group(QStringLiteral("infantry2")).readEntry("frames",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry2-versions", config.group(QStringLiteral("infantry2")).readEntry("versions",0));
 
-//   Sprites::SkinSpritesData::changeable().strData("infantry3-id", config.group("infantry3").readEntry("id"));
-  Sprites::SkinSpritesData::changeable().intData("infantry3-width", config.group("infantry3").readEntry("width",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry3-height", config.group("infantry3").readEntry("height",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry3-frames", config.group("infantry3").readEntry("frames",0));
-  Sprites::SkinSpritesData::changeable().intData("infantry3-versions", config.group("infantry3").readEntry("versions",0));
+//   Sprites::SkinSpritesData::changeable().strData("infantry3-id", config.group(QStringLiteral("infantry3")).readEntry("id"));
+  Sprites::SkinSpritesData::changeable().intData("infantry3-width", config.group(QStringLiteral("infantry3")).readEntry("width",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry3-height", config.group(QStringLiteral("infantry3")).readEntry("height",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry3-frames", config.group(QStringLiteral("infantry3")).readEntry("frames",0));
+  Sprites::SkinSpritesData::changeable().intData("infantry3-versions", config.group(QStringLiteral("infantry3")).readEntry("versions",0));
 
-//   Sprites::SkinSpritesData::changeable().strData("cavalry-id", config.group("cavalry").readEntry("id"));
-  Sprites::SkinSpritesData::changeable().intData("cavalry-width", config.group("cavalry").readEntry("width",0));
-  Sprites::SkinSpritesData::changeable().intData("cavalry-height", config.group("cavalry").readEntry("height",0));
-  Sprites::SkinSpritesData::changeable().intData("cavalry-frames", config.group("cavalry").readEntry("frames",0));
-  Sprites::SkinSpritesData::changeable().intData("cavalry-versions", config.group("cavalry").readEntry("versions",0));
+//   Sprites::SkinSpritesData::changeable().strData("cavalry-id", config.group(QStringLiteral("cavalry")).readEntry("id"));
+  Sprites::SkinSpritesData::changeable().intData("cavalry-width", config.group(QStringLiteral("cavalry")).readEntry("width",0));
+  Sprites::SkinSpritesData::changeable().intData("cavalry-height", config.group(QStringLiteral("cavalry")).readEntry("height",0));
+  Sprites::SkinSpritesData::changeable().intData("cavalry-frames", config.group(QStringLiteral("cavalry")).readEntry("frames",0));
+  Sprites::SkinSpritesData::changeable().intData("cavalry-versions", config.group(QStringLiteral("cavalry")).readEntry("versions",0));
 
-//   Sprites::SkinSpritesData::changeable().strData("cannon-id", config.group("cannon").readEntry("id"));
-  Sprites::SkinSpritesData::changeable().intData("cannon-width", config.group("cannon").readEntry("width",0));
-  Sprites::SkinSpritesData::changeable().intData("cannon-height", config.group("cannon").readEntry("height",0));
-  Sprites::SkinSpritesData::changeable().intData("cannon-frames", config.group("cannon").readEntry("frames",0));
-  Sprites::SkinSpritesData::changeable().intData("cannon-versions", config.group("cannon").readEntry("versions",0));
+//   Sprites::SkinSpritesData::changeable().strData("cannon-id", config.group(QStringLiteral("cannon")).readEntry("id"));
+  Sprites::SkinSpritesData::changeable().intData("cannon-width", config.group(QStringLiteral("cannon")).readEntry("width",0));
+  Sprites::SkinSpritesData::changeable().intData("cannon-height", config.group(QStringLiteral("cannon")).readEntry("height",0));
+  Sprites::SkinSpritesData::changeable().intData("cannon-frames", config.group(QStringLiteral("cannon")).readEntry("frames",0));
+  Sprites::SkinSpritesData::changeable().intData("cannon-versions", config.group(QStringLiteral("cannon")).readEntry("versions",0));
 
-//   Sprites::SkinSpritesData::changeable().strData("firing-id", config.group("firing").readEntry("id"));
-  Sprites::SkinSpritesData::changeable().intData("firing-width", config.group("firing").readEntry("width",0));
-  Sprites::SkinSpritesData::changeable().intData("firing-height", config.group("firing").readEntry("height",0));
-  Sprites::SkinSpritesData::changeable().intData("firing-frames", config.group("firing").readEntry("frames",0));
-  Sprites::SkinSpritesData::changeable().intData("firing-versions", config.group("firing").readEntry("versions",0));
+//   Sprites::SkinSpritesData::changeable().strData("firing-id", config.group(QStringLiteral("firing")).readEntry("id"));
+  Sprites::SkinSpritesData::changeable().intData("firing-width", config.group(QStringLiteral("firing")).readEntry("width",0));
+  Sprites::SkinSpritesData::changeable().intData("firing-height", config.group(QStringLiteral("firing")).readEntry("height",0));
+  Sprites::SkinSpritesData::changeable().intData("firing-frames", config.group(QStringLiteral("firing")).readEntry("frames",0));
+  Sprites::SkinSpritesData::changeable().intData("firing-versions", config.group(QStringLiteral("firing")).readEntry("versions",0));
 
-//   Sprites::SkinSpritesData::changeable().strData("exploding-id", config.group("exploding").readEntry("id"));
-  Sprites::SkinSpritesData::changeable().intData("exploding-width", config.group("exploding").readEntry("width",0));
-  Sprites::SkinSpritesData::changeable().intData("exploding-height", config.group("cannon").readEntry("height",0));
-  Sprites::SkinSpritesData::changeable().intData("exploding-frames", config.group("exploding").readEntry("frames",0));
-  Sprites::SkinSpritesData::changeable().intData("exploding-versions", config.group("exploding").readEntry("versions",0));
+//   Sprites::SkinSpritesData::changeable().strData("exploding-id", config.group(QStringLiteral("exploding")).readEntry("id"));
+  Sprites::SkinSpritesData::changeable().intData("exploding-width", config.group(QStringLiteral("exploding")).readEntry("width",0));
+  Sprites::SkinSpritesData::changeable().intData("exploding-height", config.group(QStringLiteral("exploding")).readEntry("height",0));
+  Sprites::SkinSpritesData::changeable().intData("exploding-frames", config.group(QStringLiteral("exploding")).readEntry("frames",0));
+  Sprites::SkinSpritesData::changeable().intData("exploding-versions", config.group(QStringLiteral("exploding")).readEntry("versions",0));
 
-  KConfigGroup fontgroup = config.group("font");
+  KConfigGroup fontgroup = config.group(QStringLiteral("font"));
   m_font.family = fontgroup.readEntry("family","URW Chancery L");
   m_font.size = fontgroup.readEntry("size",(int)(13*m_zoom));
   QString w = fontgroup.readEntry("weight", "bold");;
@@ -579,23 +570,22 @@ void ONU::buildMap()
 {
   qCDebug(KSIRK_LOG) << "with zoom="<< m_zoom;
   //QSize size((int)(m_automaton->rendererFor(m_skin).defaultSize().width()*m_zoom),(int)(m_automaton->rendererFor(m_skin).defaultSize().height()*m_zoom));
-  if (!QPixmapCache::find(m_skin+"map"+QString::number(m_width)+QString::number(m_height), &m_map))
+  const qreal dpr = qApp->devicePixelRatio();
+  const int mapDeviceWidth = m_width * dpr;
+  const int mapDeviceHeight = m_height * dpr;
+  const QString mapCacheId = m_skin + QLatin1String("map") + QString::number(mapDeviceWidth) + QLatin1Char('x') + QString::number(mapDeviceHeight);
+  if (!QPixmapCache::find(mapCacheId, &m_map))
   {
     // Pixmap isn't in the cache, create it and insert to cache
-    QSize size((int)(m_width),(int)(m_height));
-    QImage image(size, QImage::Format_ARGB32_Premultiplied);
-    image.fill(0);
-    QPainter p(&image);
-    m_automaton->rendererFor(m_skin).render(&p, "map");
-    m_map = QPixmap::fromImage(image);
-    
-    
+    m_map = QPixmap(mapDeviceWidth, mapDeviceHeight);
+    m_map.fill(Qt::transparent);
     QPainter painter(&m_map);
+    m_automaton->rendererFor(m_skin).render(&painter, "map");
+
     QFont foregroundFont(m_font.family, m_font.size, m_font.weight, m_font.italic);
     QFont backgroundFont(m_font.family, m_font.size, QFont::Normal, m_font.italic);
-    
-    painter.drawPixmap(0,0,m_map);
-    
+
+    painter.scale(dpr, dpr);
     foreach (Country* country, countries)
     {
       const QString& countryName = i18n(country->name().toUtf8().data());
@@ -623,8 +613,9 @@ void ONU::buildMap()
         int( (country->centralPoint().y()*m_zoom) + 4/*- (countryNameRect.height()/2)*/ ),
         countryName);
     }
+    m_map.setDevicePixelRatio(dpr);
 
-    QPixmapCache::insert(m_skin+"map"+QString::number(m_width)+QString::number(m_height), m_map);
+    QPixmapCache::insert(mapCacheId, m_map);
   }
 }
 
